local config = require( 'saotrace.config' )
local va = require('validate.args')

function block_setup ()

   -- just to mess things up in case the reset doesn't work
   config.test.foo = 999

   getmetatable( require('saotrace.config') ).reset()
   va.reset()
   va.opts{ check_spec = true,
	    error_on_bad_spec = true,
	 }

end

describe( "autoload config.test via config.test", function ()

  setup( block_setup )

  local test

  it( "load", function ()
    assert.has_no.error( function () test = config.test end )
  end)

  it( "check loaded value", function ()
    assert.is.equal( 3, test.foo )
  end)

  it( "set value", function ()
    assert.has_no.error( function () test.foo = 9 end )
  end)

  it( "get set value", function ()
    assert.is.equal( 9, test.foo )
  end)

end)


describe( "autoload config.test via config.test.foo", function ()

   setup( block_setup )

   local foo

   it( "load", function ()
      assert.has_no.error( function () foo = config.test.foo end )
   end)

   it( "check loaded value", function ()
     assert.is.equal( 3, foo )
   end)

end)


describe( "bad proxy", function ()

   setup( block_setup )

   it( "no module", function ()
       assert.error_matches( function () local foo = config.does_not_exist end,
			'error loading configuration'
		     )
   end)

end)

describe( "bad type", function ()

   setup( block_setup )

   it( "set", function ()
       assert.error_matches( function () config.test.foo = -1 end,
			'did not match type'
		     )
   end)

end)

describe( "read only", function ()

  before_each( block_setup )

  it( "top level", function ()
      assert.error_matches( function () config.snack= -1 end,
			'illegal assignment'
 		     )
  end)

  it( "bottom level", function ()
      assert.error_matches( function () config.test.snack= -1 end,
			'unknown element'
 		     )
  end)


end)
